#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import glob
from PyQt5 import QtCore, QtGui, QtWidgets
from cryio import ccp4image, parparser, cbfimage
from .ui.ui_wdarth import Ui_WDarth
from . import pyqt2bool


class WorkerSignals(QtCore.QObject):
    errorSignal = QtCore.pyqtSignal(str)
    progressSignal = QtCore.pyqtSignal(int)
    stopSignal = QtCore.pyqtSignal()
    finishedSignal = QtCore.pyqtSignal(object)


class Worker(QtCore.QRunnable):
    def __init__(self, params):
        QtCore.QRunnable.__init__(self)
        self.stop = False
        folder = params.pop('folder')
        self.folder = folder[:-1] if folder[-1] == '/' or folder[-1] == '\\' else folder
        self.params = params
        self.signals = WorkerSignals()
        self.signals.stopSignal.connect(self.stopIt)

    def stopIt(self):
        self.stop = True

    def parfile(self):
        self.par = parparser.ParParser(self.params['par'])

    def makemap(self):
        self.par.x = self.params['x']
        self.par.y = self.params['y']
        self.par.z = self.params['z']
        self.par.dQ = self.params['dQ']
        self.par.lorentz = self.params['lorentz']
        ccp4 = ccp4image.CCP4Image(self.par)
        for i, cbf in enumerate(self.cbflist, 1):
            if self.stop:
                break
            hdr = cbfimage.CbfHeader(cbf)
            ccp4.cbf2ccp4(hdr)
            self.signals.progressSignal.emit(i)
        if not self.stop:
            ccp4.save_map(self.mapname)
        self.signals.progressSignal.emit(len(self.cbflist))

    def run(self):
        self.basename = os.path.basename(self.folder)
        self.mapname = os.path.join(self.folder, '{}.ccp4'.format(self.basename))
        self.cbflist = glob.glob(os.path.join(self.folder, '*.cbf'))
        self.cbflist.sort()
        self.parfile()
        self.makemap()
        self.stopIt()
        self.signals.finishedSignal.emit(self)


class WorkerPool(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(int)
    finishedSignal = QtCore.pyqtSignal(list)

    def __init__(self, parent=None):
        QtCore.QObject.__init__(self, parent)
        # noinspection PyArgumentList
        self.pool = QtCore.QThreadPool.globalInstance()
        self.workers, self.errors = [], []

    def run(self, params):
        folder = params['folder']
        self.files = self.done = 0
        self.workers, self.errors = [], []
        for dirpath, dirnames, filenames in os.walk(folder):
            pars = glob.glob(os.path.join(dirpath, '*_cracker.par'))
            if not pars:
                continue
            p = params.copy()
            p['par'] = pars[0]
            p['folder'] = dirpath
            self.files += len(filenames)
            worker = Worker(p.copy())
            worker.signals.errorSignal.connect(self.workerError)
            worker.signals.progressSignal.connect(self.workerProgress)
            worker.signals.finishedSignal.connect(self.workerFinished)
            self.workers.append(worker)
        self.progressSignal.emit(0)
        for worker in self.workers:
            self.pool.start(worker)

    def stopIt(self):
        for worker in self.workers:
            worker.signals.stopSignal.emit()

    def workerProgress(self, i):
        self.done += i
        done = 100.0 * self.done / self.files
        self.progressSignal.emit(done)

    def workerError(self, folder):
        self.errors.append(folder)

    def workerFinished(self, worker):
        self.workers.remove(worker)
        if not self.workers:
            self.progressSignal.emit(100)
            self.finishedSignal.emit(self.errors)


class WDarth(QtWidgets.QDialog, Ui_WDarth):
    stopWorkerSignal = QtCore.pyqtSignal()
    runWorkerSignal = QtCore.pyqtSignal(dict)

    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        intValidator = QtGui.QIntValidator()
        intValidator.setBottom(1)
        floatValidator = QtGui.QDoubleValidator()
        floatValidator.setBottom(1e-3)
        self.xLineEdit.setValidator(intValidator)
        self.yLineEdit.setValidator(intValidator)
        self.zLineEdit.setValidator(intValidator)
        self.dQLineEdit.setValidator(floatValidator)
        self.loadSettings()
        self.stopButton.setVisible(False)
        self.workerThread = QtCore.QThread()
        self.pool = WorkerPool()
        self.pool.moveToThread(self.workerThread)
        self.pool.progressSignal.connect(self.showProgress)
        self.pool.finishedSignal.connect(self.workerFinished)
        self.stopWorkerSignal.connect(self.pool.stopIt)
        self.runWorkerSignal.connect(self.pool.run)

    def showEvent(self, event):
        self.workerThread.start()

    def showError(self, errors):
        # noinspection PyCallByClass,PyArgumentList,PyTypeChecker
        QtWidgets.QMessageBox.critical(self,
                                       'Darth Vader error',
                                       'Datasets in folders\n{}\nare corrupted: it seems you have aborted the '
                                       'experiments because the headers are empty.\n It cannot be '
                                       'processed.'.format('\n'.join(errors)))

    def showProgress(self, value):
        self.runProgressBar.setFormat('{0}: %p%'.format('Reading files'))
        self.runProgressBar.setValue(value)

    def closeEvent(self, event):
        self.saveSettings()
        self.hide()
        self.stopWorkerSignal.emit()
        self.workerThread.quit()

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('WDarth/Geometry', self.saveGeometry())
        settings.setValue('WDarth/lastFolder', self.lastFolder)
        settings.setValue('WDarth/folder', self.folderLineEdit.text())
        settings.setValue('WDarth/x', self.xLineEdit.text())
        settings.setValue('WDarth/y', self.yLineEdit.text())
        settings.setValue('WDarth/z', self.zLineEdit.text())
        settings.setValue('WDarth/dQ', self.dQLineEdit.text())
        settings.setValue('WDarth/lorentz', self.lorentzCheckBox.isChecked())

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('WDarth/Geometry', QtCore.QByteArray()))
        self.lastFolder = settings.value('WDarth/lastFolder', u'')
        self.folderLineEdit.setText(settings.value('WDarth/folder', u''))
        self.xLineEdit.setText(settings.value('WDarth/x', '256'))
        self.yLineEdit.setText(settings.value('WDarth/y', '256'))
        self.zLineEdit.setText(settings.value('WDarth/z', '256'))
        self.dQLineEdit.setText(settings.value('WDarth/dQ', '0.6'))
        self.lorentzCheckBox.setChecked(pyqt2bool(settings.value('WDarth/lorentz', False)))

    @QtCore.pyqtSlot()
    def on_runButton_clicked(self):
        folder = self.folderLineEdit.text()
        if not folder or not os.path.exists(folder):
            return
        self.runButton.setVisible(False)
        self.stopButton.setVisible(True)
        params = {
            'folder': folder,
            'x': int(self.xLineEdit.text()),
            'y': int(self.yLineEdit.text()),
            'z': int(self.zLineEdit.text()),
            'dQ': float(self.dQLineEdit.text()),
            'lorentz': int(self.lorentzCheckBox.isChecked()),
        }
        self.runWorkerSignal.emit(params)

    def workerFinished(self, errors):
        self.runButton.setVisible(True)
        self.stopButton.setVisible(False)
        if errors:
            self.showError(errors)

    @QtCore.pyqtSlot()
    def on_stopButton_clicked(self):
        self.stopWorkerSignal.emit()

    @QtCore.pyqtSlot()
    def on_folderButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        folder = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select directory', self.lastFolder)
        if not folder:
            return
        self.folderLineEdit.setText(folder)
        self.lastFolder = folder



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import division
import os
import re
import math
from multiprocessing import Pool, cpu_count
from PyQt5 import QtCore, QtGui, QtWidgets
from cryio.cbfimage import CbfHeader
# noinspection PyUnresolvedReferences
from .ui import resources_rc
from .convert import FilesPage as _FilesPage, Page, ConvertPage


REGEXP1 = re.compile(r'(\d+)p?.chi$')
REGEXP2 = re.compile(r'(\d+)p_(\d+).chi$')


def convertChi(chi, i, norm, cut, dist, h, rcy, pixel, lineEnd, scale, ifrom=0, ito=0, monFolder=''):
    folder = os.path.join(os.path.dirname(chi), 'dat')
    if not os.path.exists(folder):
        os.mkdir(folder)

    chipath = os.path.basename(chi)
    name = REGEXP1.split(chipath)
    if len(name) == 3:
        basename, period = name[:2]
        cbfname = '%s%sp.cbf' % (basename, period)
    else:
        name = REGEXP2.split(chipath)
        basename, period, frame = name[:3]
        cbfname = '%s%sp_%s.cbf' % (basename, period, frame)
    newname = '%ss%d.dat' % (basename, i)
    newnamess = '%sss%d.dat' % (basename, i)
    news = open(os.path.join(folder, newname), 'w')
    newss = open(os.path.join(folder, newnamess), 'w')
    mon = 0
    if norm == 3:
        try:
            mon = CbfHeader(os.path.join(monFolder, cbfname))['Flux']
        except (KeyError, IOError):
            mon = 1

    iscale = 0
    iscalenum = 0
    th_ii = []

    for n, line in enumerate(open(chi, 'r')):
        if n < cut or line.startswith('#'):
            continue

        th, ii = line.split()
        th, ii = float(th), float(ii)
        if ii < 0:
            ii = 0
        if norm == 2 and ifrom <= th <= ito:
            iscale += ii
            iscalenum += 1
        th_ii.append((th, ii))

    for th, ii in th_ii:
        r = dist * math.tan(math.radians(th))

        if r < h:
            try:
                alpha = 2 * math.asin(r / h)
            except ValueError:
                return ()
            l = alpha * r

        elif r == h:
            l = math.pi * r

        else:
            try:
                beta = math.acos(rcy / r)
            except ValueError:
                return ()
            alpha = 2 * (math.pi - beta)
            l = alpha * r

        nn = l / pixel
        er = math.sqrt(ii) / math.sqrt(nn)
        news.write('%f %f %f%s' % (th, ii, er, lineEnd))
        if iscale and iscalenum:
            newss.write('%.10f %.10f %.10f%s' % (th, ii / iscale * iscalenum * scale,
                                                 er / iscale * iscalenum * scale, lineEnd))
        elif mon:
            newss.write('%.10f %.10f %.10f%s' % (th, ii / mon * scale, er / mon * scale, lineEnd))

    news.close()
    newss.close()
    return folder, newname, newnamess, iscale, iscalenum


class QChiConvert(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(tuple, int, int)
    errorSignal = QtCore.pyqtSignal()
    _checkOutputSignal = QtCore.pyqtSignal(tuple)

    def __init__(self, chis, args):
        QtCore.QObject.__init__(self)
        self.args = args
        self.chis = chis
        self._checkOutputSignal.connect(self.checkOutput)

    def terminateAll(self):
        self.pool.terminate()
        self.pool.join()

    def run(self):
        ncpu = cpu_count() + 1
        self.pool = Pool(ncpu if 2 <= ncpu <= 4 else 4)
        self.queueLength = len(self.chis)
        self.processed = 0
        for i, chi in enumerate(self.chis, 1):
            args = (chi, i) + self.args
            self.pool.apply_async(convertChi, args, callback=self._checkOutputSignal.emit)
        self.pool.close()

    def checkOutput(self, tpl):
        if tpl:
            self.processed += 1
            self.progressSignal.emit(tpl, self.processed, self.queueLength)
            if self.processed == self.queueLength:
                self.pool.join()
        else:
            self.errorSignal.emit()
            self.terminateAll()


class FilesPage(_FilesPage):
    @QtCore.pyqtSlot()
    def chooseFiles(self, what='*.chi *.xy'):
        _FilesPage.chooseFiles(self, what)

    @QtCore.pyqtSlot()
    def chooseFolder(self, what='.chi .xy'):
        _FilesPage.chooseFolder(self, what)


class OptionsPage(Page):
    def setupUi(self):
        self.distSpinBox = QtWidgets.QDoubleSpinBox()
        self.distSpinBox.setMaximum(10000)
        self.distSpinBox.setMinimum(0)
        self.distSpinBox.setDecimals(2)
        layout1 = QtWidgets.QHBoxLayout()
        layout1.addWidget(QtWidgets.QLabel('Detector - sample distance (mm)'))
        layout1.addWidget(self.distSpinBox)

        self.cutSpinBox = QtWidgets.QSpinBox()
        self.cutSpinBox.setMaximum(65535)
        self.cutSpinBox.setMinimum(0)
        layout2 = QtWidgets.QHBoxLayout()
        layout2.addWidget(QtWidgets.QLabel('Number of lines to cut'))
        layout2.addWidget(self.cutSpinBox)

        self.widthSpinBox = QtWidgets.QSpinBox()
        self.widthSpinBox.setMinimum(1)
        self.widthSpinBox.setMaximum(65535)
        layout3 = QtWidgets.QHBoxLayout()
        layout3.addWidget(QtWidgets.QLabel('Detector width (pixels)'))
        layout3.addWidget(self.widthSpinBox)

        self.pixelSpinBox = QtWidgets.QDoubleSpinBox()
        self.pixelSpinBox.setMinimum(0)
        self.pixelSpinBox.setMaximum(65536)
        self.pixelSpinBox.setDecimals(5)
        layout4 = QtWidgets.QHBoxLayout()
        layout4.addWidget(QtWidgets.QLabel('Pixel size (mm)'))
        layout4.addWidget(self.pixelSpinBox)

        self.beamxSpinBox = QtWidgets.QDoubleSpinBox()
        self.beamxSpinBox.setMinimum(0)
        self.beamxSpinBox.setMaximum(65536)
        self.beamxSpinBox.setDecimals(5)
        layout5 = QtWidgets.QHBoxLayout()
        layout5.addWidget(QtWidgets.QLabel('Beam center X'))
        layout5.addWidget(self.beamxSpinBox)

        self.beamySpinBox = QtWidgets.QDoubleSpinBox()
        self.beamySpinBox.setMinimum(-65536)
        self.beamySpinBox.setMaximum(65536)
        self.beamySpinBox.setDecimals(5)
        layout6 = QtWidgets.QHBoxLayout()
        layout6.addWidget(QtWidgets.QLabel('Beam center Y'))
        layout6.addWidget(self.beamySpinBox)

        self.noRadio = QtWidgets.QRadioButton('Nothing')
        self.noRadio.setChecked(True)

        self.bgRadio = QtWidgets.QRadioButton('Background')
        self.fromSpinBox = QtWidgets.QDoubleSpinBox()
        # noinspection PyUnresolvedReferences
        self.fromSpinBox.valueChanged[float].connect(lambda _: self.bgRadio.setChecked(True))
        self.toSpinBox = QtWidgets.QDoubleSpinBox()
        # noinspection PyUnresolvedReferences
        self.toSpinBox.valueChanged[float].connect(lambda _: self.bgRadio.setChecked(True))
        layout8 = QtWidgets.QHBoxLayout()
        layout8.addWidget(self.bgRadio)
        layout8.addWidget(QtWidgets.QLabel('from'))
        layout8.addWidget(self.fromSpinBox)
        layout8.addWidget(QtWidgets.QLabel('to'))
        layout8.addWidget(self.toSpinBox)

        self.monRadio = QtWidgets.QRadioButton('Monitor')
        self.monLineEdit = QtWidgets.QLineEdit()
        openButton = QtWidgets.QPushButton(QtGui.QIcon(':/open'), '')

        # noinspection PyUnresolvedReferences,PyUnusedLocal
        @openButton.clicked.connect
        @QtCore.pyqtSlot()
        def openDir():
            _dir = os.path.dirname(self.prevPage.fileList.item(0).text())
            # noinspection PyCallByClass,PyTypeChecker
            log = QtWidgets.QFileDialog.getExistingDirectory(self, 'Choose folder with cbf files', _dir)
            self.monLineEdit.setText(log)
            self.monRadio.setChecked(True)

        layout9 = QtWidgets.QHBoxLayout()
        layout9.addWidget(self.monRadio)
        layout9.addWidget(self.monLineEdit)
        layout9.addWidget(openButton)

        layout7 = QtWidgets.QVBoxLayout()
        layout7.addWidget(self.noRadio)
        layout7.addLayout(layout8)
        layout7.addLayout(layout9)

        self.scaleLineEdit = QtWidgets.QLineEdit()
        self.scaleLineEdit.setValidator(QtGui.QIntValidator())
        layout11 = QtWidgets.QHBoxLayout()
        layout11.addWidget(QtWidgets.QLabel('Scale all files by'))
        layout11.addWidget(self.scaleLineEdit)
        layout7.addLayout(layout11)
        gb = QtWidgets.QGroupBox('Normalize by')
        gb.setLayout(layout7)

        self.winRadio = QtWidgets.QRadioButton(r'Windows (\r\n)')
        self.linRadio = QtWidgets.QRadioButton(r'Unix (\n)')
        self.macRadio = QtWidgets.QRadioButton(r'Mac (\r)')
        layout10 = QtGui.QHBoxLayout()
        layout10.addWidget(self.winRadio)
        layout10.addWidget(self.linRadio)
        layout10.addWidget(self.macRadio)
        gb1 = QtWidgets.QGroupBox('End of line')
        gb1.setLayout(layout10)

        layoutn = QtWidgets.QVBoxLayout()
        layoutn.addLayout(layout1)
        layoutn.addLayout(layout2)
        layoutn.addLayout(layout3)
        layoutn.addLayout(layout4)
        layoutn.addLayout(layout5)
        layoutn.addLayout(layout6)
        layoutn.addWidget(gb)
        layoutn.addWidget(gb1)
        layoutn.addStretch()

        self.setLayout(layoutn)

    def initializePage(self):
        settings = QtCore.QSettings()
        self.distSpinBox.setValue(float(settings.value('SigmaS/dist', 100)))
        self.cutSpinBox.setValue(int(settings.value('SigmaS/cut', 4)))
        self.widthSpinBox.setValue(int(settings.value('SigmaS/width', 1475)))
        self.pixelSpinBox.setValue(float(settings.value('SigmaS/pixel', 0.172)))
        self.beamxSpinBox.setValue(float(settings.value('SigmaS/beamx', 0)))
        self.beamySpinBox.setValue(float(settings.value('SigmaS/beamy', 0)))
        self.fromSpinBox.setValue(float(settings.value('SigmaS/from', 10)))
        self.toSpinBox.setValue(float(settings.value('SigmaS/to', 20)))
        self.monLineEdit.setText(settings.value('SigmaS/mon', ''))
        self.scaleLineEdit.setText(settings.value('SigmaS/scale', '1'))
        norm = settings.value('SigmaS/norm', '')
        if not norm:
            self.noRadio.setChecked(True)
        elif norm == 'bkg':
            self.bgRadio.setChecked(True)
        elif norm == 'mon':
            self.monRadio.setChecked(True)

        lineEnd = settings.value('SigmaS/lineend', 'win')
        if lineEnd == 'mac':
            self.macRadio.setChecked(True)
        elif lineEnd == 'unx':
            self.linRadio.setChecked(True)
        else:
            self.winRadio.setChecked(True)

    def validatePage(self):
        settings = QtCore.QSettings()
        settings.setValue('SigmaS/dist', self.distSpinBox.value())
        settings.setValue('SigmaS/cut', self.cutSpinBox.value())
        settings.setValue('SigmaS/width', self.widthSpinBox.value())
        settings.setValue('SigmaS/pixel', self.pixelSpinBox.value())
        settings.setValue('SigmaS/beamx', self.beamxSpinBox.value())
        settings.setValue('SigmaS/beamy', self.beamySpinBox.value())
        settings.setValue('SigmaS/from', self.fromSpinBox.value())
        settings.setValue('SigmaS/to', self.toSpinBox.value())
        settings.setValue('SigmaS/mon', self.monLineEdit.text())
        settings.setValue('SigmaS/scale', self.scaleLineEdit.text())
        if self.macRadio.isChecked():
            lineEnd = 'mac'
        elif self.linRadio.isChecked():
            lineEnd = 'unx'
        else:
            lineEnd = 'win'
        settings.setValue('SigmaS/lineend', lineEnd)
        if self.bgRadio.isChecked():
            norm = 'bkg'
        elif self.monRadio.isChecked():
            norm = 'mon'
        else:
            norm = ''
        settings.setValue('SigmaS/norm', norm)
        return Page.validatePage(self)


class SigmaSWizard(QtWidgets.QWizard):
    terminateSignal = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        QtWidgets.QWizard.__init__(self, parent)
        filesPage = FilesPage()
        self.addPage(filesPage)
        self.addPage(OptionsPage(prevPage=filesPage))
        self.addPage(ConvertPage())
        self.setWindowTitle('Pylatus Sigma Scale')
        self.setOptions(QtWidgets.QWizard.NoBackButtonOnLastPage)
        self.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        matrix = QtGui.QTransform().rotate(-90)
        pixmap = QtGui.QPixmap(':/snbl').scaled(281, 151, QtCore.Qt.KeepAspectRatio,
                                                QtCore.Qt.SmoothTransformation).transformed(matrix)
        self.setPixmap(QtWidgets.QWizard.WatermarkPixmap, pixmap)
        self.loadSettings()
        self.setWindowIcon(QtGui.QIcon(':/scale'))
        self.stopped = False
        self.count = 1
        # noinspection PyUnresolvedReferences
        self.currentIdChanged[int].connect(self.checkPage)
        self.rejected.connect(self.terminate)
        self.workerThread = QtCore.QThread()
        self.errorShown = False

    def terminate(self):
        self.terminateSignal.emit()
        self.finish()

    def checkPage(self, page):
        if page == 2:
            self.convert()

    def convert(self):
        page1 = self.page(1)
        dist = page1.distSpinBox.value()
        cut = page1.cutSpinBox.value()
        pixel = page1.pixelSpinBox.value()
        beamy = page1.beamySpinBox.value()
        beamx = page1.beamxSpinBox.value()
        rcy = beamy * pixel
        h = beamx * pixel

        if page1.macRadio.isChecked():
            self.lineEnd = '\r'
        elif page1.linRadio.isChecked():
            self.lineEnd = '\n'
        else:
            self.lineEnd = '\r\n'

        norm = ifrom = ito = 1
        monFolder = ''
        if page1.bgRadio.isChecked():
            ifrom = page1.fromSpinBox.value()
            ito = page1.toSpinBox.value()
            norm = 2
        elif page1.monRadio.isChecked():
            norm = 3
            monFolder = page1.monLineEdit.text()

        scale = int(page1.scaleLineEdit.text() or 1)
        page0 = self.page(0)
        count = page0.fileList.count()
        self.scale, self.bufs, self.bufss = [], [], []
        chis = [page0.fileList.item(i).text() for i in range(count)]
        args = norm, cut, dist, h, rcy, pixel, self.lineEnd, scale, ifrom, ito, monFolder,
        self.worker = QChiConvert(chis, args)
        self.worker.moveToThread(self.workerThread)
        # noinspection PyUnresolvedReferences
        self.workerThread.started.connect(self.worker.run)
        self.terminateSignal.connect(self.worker.terminateAll)
        self.worker.progressSignal.connect(self.showProgress)
        self.worker.errorSignal.connect(self.showError)
        self.workerThread.start()

    def showProgress(self, results, processed, queueLength):
        folder, newname, newnamess, iscale, iscalenum = results
        self.bufs.append(newname)
        self.bufss.append(newnamess)
        self.scale.append((newnamess, iscale, iscalenum))
        page2 = self.page(2)
        page2.log.addItem('-> dat/%s' % newname)
        page2.log.scrollToBottom()
        page2.progress.setValue(100 * (processed + 1.) / queueLength)

        if processed == queueLength:
            exp = re.compile(r'(\d+).dat$')
            self.scale.sort(key=lambda k: int(exp.split(k[0])[1]))
            self.bufs.sort(key=lambda k: int(exp.split(k)[1]))
            self.bufss.sort(key=lambda k: int(exp.split(k)[1]))
            open(os.path.join(folder, 'scalers.dat'), 'w').write(self.lineEnd.join(
                ('%s %f %f' % s for s in self.scale)))
            open(os.path.join(folder, 'buffers.buf'), 'w').write(self.lineEnd.join(self.bufs))
            open(os.path.join(folder, 'bufferss.buf'), 'w').write(self.lineEnd.join(self.bufss))
            self.finish()

    def finish(self):
        self.stopped = True
        self.page(2).completeChanged.emit()
        self.workerThread.quit()
        self.workerThread.wait()

    def showError(self):
        if self.errorShown:
            return
        self.errorShown = True
        self.finish()
        # noinspection PyCallByClass,PyTypeChecker,PyArgumentList
        QtWidgets.QMessageBox.critical(self, 'Error in thread', 'It seems you have to swap beamX and beamY values')

    def closeEvent(self, event):
        self.saveSettings()
        event.accept()

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('SigmaS/Geometry', QtCore.QByteArray()))

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('SigmaS/Geometry', self.saveGeometry())



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from PyQt5 import QtCore, QtGui, QtWidgets
from cryio import cbfimage
from .ui.ui_wheader import Ui_WHeader


class WHeader(QtWidgets.QDialog, Ui_WHeader):
    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.valuesList.addItems(sorted(list(cbfimage.CbfImage.ALL_KEYWORDS.keys()) + ['!File name', 'Date', 'Time']))
        self.loadSettings()
        self.stopButton.setVisible(False)
        self.stopped = True
        self.createContextMenu()
        self.resultTable.keyPressEvent = self.resultTableKeyPressEvent
        self.resultTable.contextMenuEvent = self.resultTableContextMenuEvent

    def createContextMenu(self):
        copyMenuAction = QtWidgets.QAction('&Copy', self.resultTable)
        copyMenuAction.setShortcut(QtGui.QKeySequence.Copy)
        # noinspection PyUnresolvedReferences
        copyMenuAction.triggered.connect(self.copyToClipboard)
        saveTextAction = QtWidgets.QAction('&Save as text', self.resultTable)
        saveTextAction.setShortcut(QtGui.QKeySequence.Save)
        # noinspection PyUnresolvedReferences
        saveTextAction.triggered.connect(self.on_saveButton_clicked)
        self.contextMenu = QtWidgets.QMenu()
        self.contextMenu.addAction(copyMenuAction)
        self.contextMenu.addAction(saveTextAction)

    def resultTableContextMenuEvent(self, event):
        self.contextMenu.exec_(event.globalPos())

    @QtCore.pyqtSlot()
    def copyToClipboard(self):
        selectedItems = self.resultTable.selectedItems()
        toClipboard = u''
        for i in range(self.resultTable.rowCount()):
            setNewLine = u''
            for j in range(self.resultTable.columnCount()):
                item = self.resultTable.item(i, j)
                if selectedItems:
                    if item in selectedItems:
                        toClipboard += item.text() + u'\t'
                        setNewLine = '\n'
                elif item:
                    toClipboard += item.text() + u'\t'
                    setNewLine = '\n'
            toClipboard += setNewLine
        # noinspection PyArgumentList
        QtWidgets.QApplication.clipboard().setText(toClipboard)

    def resultTableKeyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_C and event.modifiers() == QtCore.Qt.ControlModifier:
            self.copyToClipboard()

    def closeEvent(self, event):
        self.saveSettings()
        self.hide()

    def checkedHeaderValues(self):
        checked = []
        for i in range(self.valuesList.count()):
            item = self.valuesList.item(i)
            if item.checkState():
                checked.append(item.text())
        return checked

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('WHeader/Geometry', self.saveGeometry())
        settings.setValue('WHeader/checked', self.checkedHeaderValues())
        settings.setValue('WHeader/lastFolder', self.lastFolder)

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('WHeader/Geometry', QtCore.QByteArray()))
        self.lastFolder = settings.value('WHeader/lastFolder', u'')
        self.folderLineEdit.setText(self.lastFolder)
        checked = settings.value('WHeader/checked') or []
        for i in range(self.valuesList.count()):
            item = self.valuesList.item(i)
            item.setFlags(QtCore.Qt.ItemIsUserCheckable | QtCore.Qt.ItemIsEnabled)
            item.setCheckState(QtCore.Qt.Checked if item.text() in checked else QtCore.Qt.Unchecked)
        self.on_valuesList_itemClicked()

    #noinspection PyUnusedLocal
    def on_valuesList_itemClicked(self, item=None):
        self.resultTable.clear()
        self.resultTable.setRowCount(0)
        checked = self.checkedHeaderValues()
        self.resultTable.setColumnCount(len(checked))
        self.resultTable.setHorizontalHeaderLabels(checked)

    @QtCore.pyqtSlot()
    def on_stopButton_clicked(self):
        self.stopped = True

    @QtCore.pyqtSlot()
    def on_runButton_clicked(self):
        self.stopped = False
        self.runButton.setVisible(False)
        self.stopButton.setVisible(True)
        self.walkThroughFolder()
        self.stopped = True
        self.runButton.setVisible(True)
        self.stopButton.setVisible(False)

    @QtCore.pyqtSlot()
    def on_saveButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        datName = QtWidgets.QFileDialog.getSaveFileName(self, 'Save file as a text', self.lastFolder)
        if not datName:
            return
        res = u'%s\n' % u' '.join(self.checkedHeaderValues())
        for i in range(self.resultTable.rowCount()):
            for j in range(self.resultTable.columnCount()):
                res += u'%s ' % self.resultTable.item(i, j).text()
            res += '\n'
        open(datName, 'w').write(res)

    def walkThroughFolder(self):
        self.on_valuesList_itemClicked()
        folder = self.folderLineEdit.text()
        checked = self.checkedHeaderValues()
        if not folder or not checked or not os.path.exists(folder):
            return
        files = sorted([f for f in os.listdir(folder) if f.endswith('.cbf')])
        self.resultTable.setRowCount(len(files))
        for i, cbf in enumerate(files):
            # noinspection PyArgumentList
            QtCore.QCoreApplication.processEvents()
            if self.stopped:
                return

            hdr = cbfimage.CbfHeader(os.path.join(folder, cbf))
            for j, key in enumerate(checked):
                try:
                    value = hdr[key]
                except KeyError:
                    continue
                item = QtWidgets.QTableWidgetItem(str(value))
                self.resultTable.setItem(i, j, item)
            self.runProgressBar.setValue(100 * (i + 1) / len(files))

    @QtCore.pyqtSlot()
    def on_folderButton_clicked(self):
        # noinspection PyCallByClass,PyTypeChecker
        folder = QtWidgets.QFileDialog.getExistingDirectory(self, 'Choose folder with cbf files', self.lastFolder)
        if folder:
            self.folderLineEdit.setText(folder)
            self.lastFolder = folder

    @QtCore.pyqtSlot()
    def on_copyButton_clicked(self):
        self.resultTable.selectAll()
        self.copyToClipboard()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import re
import copy
import sip
import multiprocessing as mp
import six
from PyQt5 import QtCore, QtGui, QtWidgets
import cryio
from cryio.cbfimage import CbfImage
from . import pyqt2bool
# noinspection PyUnresolvedReferences
from .ui import resources_rc


NEGATIVE_THRESHOLD = -3


def convertCbf(opts):
    folder = os.path.join(os.path.dirname(opts['images'][0]), opts['format'])
    angleInc = 0
    if six.PY3:
        try:
            os.mkdir(folder)
        except FileExistsError:
            pass
    else:
        try:
            os.mkdir(folder)
        except OSError:
            pass
    img = cryio.openImage(opts['images'][0])
    if isinstance(img, CbfImage):
        angleInc = img.header_dict['Angle_increment']
    for image in opts['images'][1:]:
        img += cryio.openImage(image)
    if opts['negative'] > NEGATIVE_THRESHOLD:
        img.array[img.array < 0] = opts['negative']
    if isinstance(img, CbfImage) and 'changeAngleIncrement' in opts and not opts['changeAngleIncrement']:
        img.header_dict['Angle_increment'] = angleInc
    if 'Pixel_size' in img.header:
        img.header['pixel_size'] = img.header['Pixel_size'][0]
    opts.update(img.header)
    newImg = os.path.join(folder, opts['name'])
    img.save(newImg, opts['format'], **opts)
    return '-> %s/%s' % (opts['format'], opts['name'])


class QCbfConvert(QtCore.QObject):
    progressSignal = QtCore.pyqtSignal(str, int, int)
    poolSignal = QtCore.pyqtSignal(str)

    def __init__(self, opts):
        QtCore.QObject.__init__(self)
        self.opts = opts
        self.poolSignal.connect(self.showProgress)

    def terminateAll(self):
        self.pool.terminate()
        self.pool.join()

    def simpleConversion(self):
        for cbf in self.opts['images']:
            name = '%s.%s' % (os.path.basename(cbf)[:-4], self.opts['format'])
            nargs = copy.deepcopy(self.opts)
            nargs.update({'images': (cbf,), 'name': name})
            yield nargs

    def makeChunk(self, i, images):
        args = copy.deepcopy(self.opts)
        name = '%s_%05d.%s' % ('_'.join(os.path.basename(images[0])[:-4].split('_')[:-1]), i, self.opts['format'])
        args.update({'images': images, 'name': name})
        return args

    def binFrames(self):
        i, images = 0, []
        for j, cbf in enumerate(self.opts['images']):
            images.append(cbf)
            if len(images) == self.opts['bintype']:
                i += 1
                chunk = self.makeChunk(i, images)
                yield chunk
                images = []
        converted = i * self.opts['bintype']
        if converted < len(self.opts['images']):
            i += 1
            yield self.makeChunk(i, self.opts['images'][converted:])

    def binPeriods(self):
        pattern = re.compile(r'.+_(\d+)p_(\d+)\.cbf$')
        self.opts['changeAngleIncrement'] = False

        def keyfunc(cbf):
            period, frame = map(int, pattern.split(cbf)[1:3])
            if not keyfunc.i:
                keyfunc.i = period
            if period > self.opts['bintype']:
                self.opts['bintype'] = period
            return frame
        keyfunc.i = 0

        try:
            self.opts['images'].sort(key=keyfunc)
        except KeyError:
            pass
        if keyfunc.i != 1:
            self.opts['bintype'] -= keyfunc.i - 1
        return self.binFrames()

    def convert(self):
        ncpu = mp.cpu_count()
        self.pool = mp.Pool(ncpu + 1 if 1 <= ncpu <= 5 else 5)
        if self.opts['binning']:
            if self.opts['bintype']:
                images = self.binFrames()
            else:
                images = self.binPeriods()
        else:
            images = self.simpleConversion()

        images = list(images)
        self.queueLength, self.converted = len(images), 0
        for chunk in images:
            if six.PY3:
                self.pool.apply_async(convertCbf, (chunk,), callback=self.poolSignal.emit,
                                      error_callback=self.showError)
            else:
                self.pool.apply_async(convertCbf, (chunk,), callback=self.poolSignal.emit)
            # self.poolSignal.emit(convertCbf(chunk))
        self.pool.close()

    def showError(self, err):
        print(err)

    def showProgress(self, fname):
        self.converted += 1
        self.progressSignal.emit(fname, self.converted, self.queueLength)
        if self.converted == self.queueLength:
            self.pool.join()


class Page(QtWidgets.QWizardPage):
    def __init__(self, parent=None, prevPage=None):
        QtWidgets.QWizardPage.__init__(self, parent)
        self.prevPage = prevPage
        self.setupUi()

    def setupUi(self):
        pass


class FilesPage(Page):
    # noinspection PyUnresolvedReferences
    def setupUi(self):
        folderButton = QtGui.QPushButton('Choose folder...')
        folderButton.clicked.connect(self.chooseFolder)

        filesButton = QtGui.QPushButton('Choose files...')
        filesButton.clicked.connect(self.chooseFiles)

        label = QtGui.QLabel('or')

        hlayout = QtGui.QHBoxLayout()
        hlayout.addWidget(folderButton)
        hlayout.addWidget(label)
        hlayout.addWidget(filesButton)
        hlayout.addStretch()

        self.fileList = QtGui.QListWidget()
        self.fileList.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        delButton = QtGui.QPushButton('Delete items')
        delButton.clicked.connect(self.delItems)

        clearButton = QtGui.QPushButton('Clear')
        clearButton.clicked.connect(self.clearItems)

        self.label = QtGui.QLabel('Selected files: 0')

        hlayout1 = QtGui.QHBoxLayout()
        hlayout1.addWidget(self.label)
        hlayout1.addStretch()
        hlayout1.addWidget(delButton)
        hlayout1.addWidget(clearButton)

        vlayout = QtGui.QVBoxLayout()
        vlayout.addLayout(hlayout)
        vlayout.addWidget(self.fileList)
        vlayout.addLayout(hlayout1)
        self.setLayout(vlayout)

    def initializePage(self):
        settings = QtCore.QSettings()
        self.lastFolder = settings.value('Convert/lastFolder', '')

    @QtCore.pyqtSlot()
    def chooseFolder(self, what='.cbf .edf'):
        # noinspection PyCallByClass,PyTypeChecker,PyArgumentList
        d = QtWidgets.QFileDialog.getExistingDirectory(self, 'Choose the folder with %s files' % what, self.lastFolder)
        if d:
            files = []
            for dirpath, dirnames, filenames in os.walk(d):
                for ext in what.split(' '):
                    files += [os.path.join(dirpath, f) for f in filenames if f.endswith(ext)]
                    # noinspection PyArgumentList
                    QtCore.QCoreApplication.processEvents()
            self.openFiles(files)

    @QtCore.pyqtSlot()
    def chooseFiles(self, what='*.cbf *.edf'):
        # noinspection PyCallByClass,PyTypeChecker,PyArgumentList
        files = QtWidgets.QFileDialog.getOpenFileNames(self, 'Choose %s files' % what,
                                                       self.lastFolder, 'Pilatus data (%s)' % what)[0]
        self.openFiles(files)

    def openFiles(self, files):
        if not files:
            return

        files.sort()
        self.lastFolder = os.path.dirname(files[0])
        self.fileList.addItems(files)
        self.label.setText('Selected files: %d' % self.fileList.count())

    @QtCore.pyqtSlot()
    def delItems(self):
        for item in self.fileList.selectedItems():
            sip.delete(item)
        self.label.setText('Selected files: %d' % self.fileList.count())

    @QtCore.pyqtSlot()
    def clearItems(self):
        self.fileList.clear()
        self.label.setText('Selected files: %d' % self.fileList.count())

    def validatePage(self):
        settings = QtCore.QSettings()
        settings.setValue('Convert/lastFolder', self.lastFolder)
        return Page.validatePage(self)


class OptionsPage(Page):
    # noinspection PyUnresolvedReferences
    def setupUi(self):
        self.cbf = QtGui.QRadioButton('cbf')
        self.cbf.setChecked(True)
        self.edf = QtGui.QRadioButton('edf')

        hrlayout = QtGui.QHBoxLayout()
        vrlayout1 = QtGui.QVBoxLayout()
        vrlayout1.addWidget(self.cbf)
        vrlayout2 = QtGui.QVBoxLayout()
        vrlayout2.addWidget(self.edf)
        hrlayout.addLayout(vrlayout1)
        hrlayout.addLayout(vrlayout2)

        radiosGroupBox = QtGui.QGroupBox('Choose the format to convert')
        radiosGroupBox.setLayout(hrlayout)

        self.binningCheckBox = QtGui.QCheckBox('Make binning')
        self.binningEveryRadio = QtGui.QRadioButton('for every')
        self.binningSpinBox = QtGui.QSpinBox()
        self.binningSpinBox.setDisabled(True)
        self.binningSpinBox.setMinimum(2)
        self.binningSpinBox.setMaximum(65535)
        self.binningPeriodRadio = QtGui.QRadioButton('periodically')
        filesLabel = QtGui.QLabel('files')
        self.binningGroup = QtGui.QButtonGroup()
        self.binningGroup.addButton(self.binningEveryRadio)
        self.binningGroup.addButton(self.binningPeriodRadio)

        def toggleBinning(enable=None):
            if enable is None:
                enable = self.binningCheckBox.isChecked()
            self.binningSpinBox.setEnabled(enable)
            self.binningEveryRadio.setEnabled(enable)
            self.binningPeriodRadio.setEnabled(enable)
            filesLabel.setEnabled(enable)

        toggleBinning(False)
        self.binningEveryRadio.setChecked(True)
        self.binningCheckBox.toggled.connect(toggleBinning)
        self.binningPeriodRadio.toggled.connect(
            lambda: self.binningSpinBox.setDisabled(self.binningPeriodRadio.isChecked())
        )

        hlayout1 = QtGui.QHBoxLayout()
        hlayout1.addWidget(self.binningCheckBox)
        hlayout1.addWidget(self.binningEveryRadio)
        hlayout1.addWidget(self.binningSpinBox)
        hlayout1.addWidget(filesLabel)
        hlayout1.addWidget(self.binningPeriodRadio)
        hlayout1.addStretch()

        self.negativeCheckBox = QtGui.QCheckBox('Set negative intensity as')
        self.negativeSpinBox = QtGui.QSpinBox()
        self.negativeSpinBox.setMinimum(-2)
        self.negativeSpinBox.setDisabled(True)
        self.negativeCheckBox.clicked.connect(
            lambda: self.negativeSpinBox.setEnabled(self.negativeCheckBox.isChecked())
        )

        hlayout2 = QtGui.QHBoxLayout()
        hlayout2.addWidget(self.negativeCheckBox)
        hlayout2.addWidget(self.negativeSpinBox)
        hlayout2.addStretch()

        self.moveAngleCheckBox = QtGui.QCheckBox('Move angle:')
        self.startingAngleSpinBox = QtGui.QDoubleSpinBox()
        self.startingAngleSpinBox.setDecimals(4)
        self.startingAngleSpinBox.setMinimum(-360)
        self.startingAngleSpinBox.setMaximum(360)
        self.angleIncrementSpinBox = QtGui.QDoubleSpinBox()
        self.angleIncrementSpinBox.setDecimals(4)
        self.angleIncrementSpinBox.setMinimum(-360)
        self.angleIncrementSpinBox.setMaximum(360)
        startLabel = QtGui.QLabel('start')
        incLabel = QtGui.QLabel('increment')
        hlayout3 = QtGui.QHBoxLayout()
        hlayout3.addWidget(self.moveAngleCheckBox)
        hlayout3.addWidget(startLabel)
        hlayout3.addWidget(self.startingAngleSpinBox)
        hlayout3.addWidget(incLabel)
        hlayout3.addWidget(self.angleIncrementSpinBox)
        hlayout3.addStretch()

        def showAngle(show):
            self.moveAngleCheckBox.setVisible(show)
            self.startingAngleSpinBox.setVisible(show)
            self.angleIncrementSpinBox.setVisible(show)
            startLabel.setVisible(show)
            incLabel.setVisible(show)

        showAngle(True)
        self.cbf.toggled[bool].connect(showAngle)

        def enableAngle(en):
            self.angleIncrementSpinBox.setEnabled(en)
            self.startingAngleSpinBox.setEnabled(en)

        enableAngle(False)
        self.moveAngleCheckBox.toggled[bool].connect(enableAngle)

        self.int32 = QtGui.QRadioButton('Int32 (for fit2d)')
        self.int32.setChecked(True)
        self.int16 = QtGui.QRadioButton('Int16')
        self.intGroup = QtGui.QButtonGroup()
        self.intGroup.addButton(self.int32)
        self.intGroup.addButton(self.int16)
        self.flipCheckBox = QtGui.QCheckBox('Flip image (for fit2d)')
        self.flipCheckBox.setChecked(True)
        self.addHeaderCheckBox = QtGui.QCheckBox('Add edf header (for fit2d)')
        self.addHeaderCheckBox.setChecked(True)
        hlayout4 = QtGui.QHBoxLayout()
        hlayout4.addWidget(self.int32)
        hlayout4.addWidget(self.int16)
        hlayout4.addStretch()
        hlayout5 = QtGui.QHBoxLayout()
        hlayout5.addWidget(self.flipCheckBox)
        hlayout5.addStretch()
        hlayout6 = QtGui.QHBoxLayout()
        hlayout6.addWidget(self.addHeaderCheckBox)
        hlayout6.addStretch()

        def showEdfOpts(show):
            self.int16.setVisible(show)
            self.int32.setVisible(show)
            self.flipCheckBox.setVisible(show)
            self.addHeaderCheckBox.setVisible(show)

        showEdfOpts(False)
        self.edf.toggled[bool].connect(showEdfOpts)

        vlayout1 = QtGui.QVBoxLayout()
        vlayout1.addLayout(hlayout1)
        vlayout1.addLayout(hlayout2)
        vlayout1.addLayout(hlayout3)
        vlayout1.addLayout(hlayout4)
        vlayout1.addLayout(hlayout5)
        vlayout1.addLayout(hlayout6)

        optionsGroupBox = QtGui.QGroupBox('Options')
        optionsGroupBox.setLayout(vlayout1)

        vlayout2 = QtGui.QVBoxLayout()
        vlayout2.addWidget(radiosGroupBox)
        vlayout2.addWidget(optionsGroupBox)
        self.setLayout(vlayout2)

    def initializePage(self):
        settings = QtCore.QSettings()
        self.setRadio(settings.value('Convert/format', 'cbf'), 'edf', 'cbf')
        binning = pyqt2bool(settings.value('Convert/binning', False))
        self.binningCheckBox.setChecked(binning)
        self.binningSpinBox.setValue(int(settings.value('Convert/binning_value', 1)))
        self.setRadio(settings.value('Convert/bintype', 'binningEveryRadio'), 'binningEveryRadio', 'binningPeriodRadio')
        negative = pyqt2bool(settings.value('Convert/negative', False))
        self.negativeCheckBox.setChecked(negative)
        self.negativeSpinBox.setEnabled(negative)
        self.negativeSpinBox.setValue(int(settings.value('Convert/negative_value', 0)))
        self.angleIncrementSpinBox.setValue(float(settings.value('Convert/angleIncrement', 0)))
        self.startingAngleSpinBox.setValue(float(settings.value('Convert/startingAngle', 0)))
        self.flipCheckBox.setChecked(pyqt2bool(settings.value('Convert/edfFlip', True)))
        self.setRadio(settings.value('Convert/edfInt', 'int32'), 'int16', 'int32')
        self.addHeaderCheckBox.setChecked(pyqt2bool(settings.value('Convert/edfAddHeader', True)))

    def getRadio(self, *radios):
        for item in radios:
            if self.__dict__[item].isChecked():
                return item

    def setRadio(self, value, *radios):
        for item in radios:
            if value == item:
                self.__dict__[item].setChecked(True)
                return

    def validatePage(self):
        settings = QtCore.QSettings()
        settings.setValue('Convert/format', self.getRadio('cbf', 'edf'))
        settings.setValue('Convert/binning', self.binningCheckBox.isChecked())
        settings.setValue('Convert/bintype', self.getRadio('binningEveryRadio', 'binningPeriodRadio'))
        settings.setValue('Convert/binning_value', self.binningSpinBox.value())
        settings.setValue('Convert/negative', self.negativeCheckBox.isChecked())
        settings.setValue('Convert/negative_value', self.negativeSpinBox.value())
        settings.setValue('Convert/startingAngle', self.startingAngleSpinBox.value())
        settings.setValue('Convert/angleIncrement', self.angleIncrementSpinBox.value())
        settings.setValue('Convert/edfInt', self.getRadio('int16', 'int32'))
        settings.setValue('Convert/edfFlip', self.flipCheckBox.isChecked())
        settings.setValue('Convert/edfAddHeader', self.addHeaderCheckBox.isChecked())
        return Page.validatePage(self)


class ConvertPage(Page):
    def setupUi(self):
        self.progress = QtWidgets.QProgressBar()
        self.progress.setMaximum(100)
        self.log = QtWidgets.QListWidget()

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.progress)
        layout.addWidget(self.log)
        self.setLayout(layout)

    def isComplete(self):
        return self.wizard().stopped


class ConvertWizard(QtWidgets.QWizard):
    terminateSignal = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        QtWidgets.QWizard.__init__(self, parent)
        self.addPage(FilesPage())
        self.addPage(OptionsPage())
        self.addPage(ConvertPage())
        self.setWindowTitle('Pylatus image converter')
        self.setOptions(QtWidgets.QWizard.NoBackButtonOnLastPage)
        self.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        matrix = QtGui.QTransform().rotate(-90)
        pixmap = QtGui.QPixmap(':/snbl').scaled(281, 151, QtCore.Qt.KeepAspectRatio,
                                                QtCore.Qt.SmoothTransformation).transformed(matrix)
        self.setPixmap(QtWidgets.QWizard.WatermarkPixmap, pixmap)
        self.loadSettings()
        self.setWindowIcon(QtGui.QIcon(':/convert'))
        # noinspection PyUnresolvedReferences
        self.currentIdChanged[int].connect(self.checkPage)
        self.stopped = False
        self.workerThread = QtCore.QThread()
        # noinspection PyUnresolvedReferences
        self.rejected.connect(self.terminate)

    def terminate(self):
        self.terminateSignal.emit()
        self.finished()

    def checkPage(self, page):
        if page == 2:
            self.convert()

    def closeEvent(self, event):
        self.saveSettings()
        event.accept()

    def loadSettings(self):
        settings = QtCore.QSettings()
        self.restoreGeometry(settings.value('Convert/Geometry', QtCore.QByteArray()))

    def saveSettings(self):
        settings = QtCore.QSettings()
        settings.setValue('Convert/Geometry', self.saveGeometry())

    def showProgress(self, images, converted, queueLength):
        self.log.addItem(images)
        self.log.scrollToBottom()
        self.progress.setValue(100.0 * converted / queueLength)
        if converted == queueLength:
            self.finished()

    def finished(self, p_int=0):
        self.stopped = True
        if hasattr(self, 'page2'):
            self.page2.completeChanged.emit()
        self.workerThread.quit()
        self.workerThread.wait()

    def convert(self):
        self.page2 = self.page(2)
        self.progress = self.page2.progress
        self.log = self.page2.log

        fileList = self.page(0).fileList
        count = fileList.count()

        page1 = self.page(1)
        if page1.negativeCheckBox.isChecked():
            negative = page1.negativeSpinBox.value()
        else:
            negative = NEGATIVE_THRESHOLD

        self.format = page1.getRadio('cbf', 'edf')
        if page1.moveAngleCheckBox.isChecked():
            move = [page1.startingAngleSpinBox.value(), page1.angleIncrementSpinBox.value()]
        else:
            move = []
        images = [fileList.item(i).text() for i in range(count)]

        opts = {
            'images': images,
            'format': self.format,
            'negative': negative,
            'binning': page1.binningCheckBox.isChecked(),
            'bintype': page1.binningSpinBox.value() if page1.binningSpinBox.isEnabled() else 0,
            'move': move,
            'edfInt': page1.getRadio('int16', 'int32'),
            'edfFlip': page1.flipCheckBox.isChecked(),
            'edfHeader': page1.addHeaderCheckBox.isChecked(),
        }
        self.worker = QCbfConvert(copy.deepcopy(opts))
        self.worker.moveToThread(self.workerThread)
        # noinspection PyUnresolvedReferences
        self.workerThread.started.connect(self.worker.convert)
        self.worker.progressSignal.connect(self.showProgress)
        self.terminateSignal.connect(self.worker.terminateAll)
        self.workerThread.start()
